#!/usr/bin/perl -w
#####
#
#   svg_via_dom.pl
#   written by Kevin Lindsey (kevin@kevlindev.com)
#
#####

#####
#
#   includes
#
#####
use QOTDCommon;
use XML::DOM;


#####
#
#   globals
#
#####
my $svgDocument;
my $svgRoot;


#####
#
#   main
#
#####
create_SVG_Document();
$svgRoot->appendChild( create_style() );
$svgRoot->appendChild( create_defs()  );
$svgRoot->appendChild( create_quote() );

print "Pragma: no-cache\n";
print "Content-type: image/svg+xml\n\n";
print $svgDocument->toString();


#####
#
#   create_SVG_Document
#
#####
sub create_SVG_Document {
    $svgDocument = new XML::DOM::Document();

    my $xmlDecl = new XML::DOM::XMLDecl($svgDocument, "1.0", "ISO-8859-1", 0);
    my $doctype = new XML::DOM::DocumentType($svgDocument);

    $svgDocument->setXMLDecl($xmlDecl);

    $doctype->setParams(
        "svg",
        "http://www.w3.org/TR/2001/REC-SVG-200100904/DTD/svg10.dtd",
        "-//W3C//DTD SVG 20010904//EN"
    );
    $svgDocument->setDoctype($doctype);

    $svgRoot = $svgDocument->createElement("svg");
    $svgRoot->setAttribute("xmlns", "http://www.w3.org/2000/svg");
    $svgRoot->setAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
    $svgDocument->appendChild($svgRoot);
}


#####
#
#   create_style
#
#####
sub create_style {
    my $style = $svgDocument->createElement("style");
    my $cdata = $svgDocument->createCDATASection(<<EOL);
    .quote { font-size: 28; font-weight: bold }
    .author { font-size: 24; font-weight: normal; font-style: italic }
EOL
    
    $style->setAttribute("type", "text/css");
    $style->appendChild($cdata);

    return $style;
}


#####
#
#   create_defs
#
#####
sub create_defs {
    my $defs = $svgDocument->createElement("defs");
    
    $defs->appendChild( create_tile_element() );
    $defs->appendChild( create_cluster()      );
    $defs->appendChild( create_supercluster() );
    $defs->appendChild( create_pattern()      );
    $defs->appendChild( create_filter()       );

    return $defs;
}


#####
#
#   create_tile_element
#
#####
sub create_tile_element {
    my $elem = $svgDocument->createElement("path");

    $elem->setAttribute("id",           "elem");
    $elem->setAttribute("d",            $path_data);
    $elem->setAttribute("stroke",       $path_stroke_color);
    $elem->setAttribute("stroke-width", $path_stroke_width);
    $elem->setAttribute("fill",         $path_fill_color);
    $elem->setAttribute("fill-opacity", $path_fill_opacity);
    $elem->setAttribute("transform",    $path_transform);

    return $elem;
}


#####
#
#   create_cluster
#
#####
sub create_cluster {
    my $g    = $svgDocument->createElement("g");
    my $use1 = $svgDocument->createElement("use");
    my $use2 = $svgDocument->createElement("use");
    my $use3 = $svgDocument->createElement("use");

    $g->setAttribute("id", "cluster");

    $use1->setAttribute("xlink:href", "#elem");
    $use1->setAttribute("transform", "rotate(0)");
    $g->appendChild($use1);

    $use2->setAttribute("xlink:href", "#elem");
    $use2->setAttribute("transform", "rotate(120)");
    $g->appendChild($use2);

    $use3->setAttribute("xlink:href", "#elem");
    $use3->setAttribute("transform", "rotate(240)");
    $g->appendChild($use3);

    return $g;
}


#####
#
#   create_supercluster
#
#####
sub create_supercluster {
    my $g    = $svgDocument->createElement("g");
    my $use1 = $svgDocument->createElement("use");
    my $use2 = $svgDocument->createElement("use");
    my $use3 = $svgDocument->createElement("use");

    $g->setAttribute("id", "supercluster");

    $use1->setAttribute("xlink:href", "#cluster");
    $use1->setAttribute(
        "transform",
        "rotate(60) translate($hexagon_radius,0) rotate(60) scale(-1,1)"
    );
    $g->appendChild($use1);

    $use2->setAttribute("xlink:href", "#cluster");
    $use2->setAttribute(
        "transform",
        "rotate(0) translate($hexagon_radius,0)"
    );
    $g->appendChild($use2);

    $use3->setAttribute("xlink:href", "#cluster");
    $use3->setAttribute(
        "transform",
        "rotate(-60) translate($hexagon_radius,0) scale(1,-1)"
    );
    $g->appendChild($use3);

    return $g;
}


#####
#
#   create_pattern
#
#####
sub create_pattern {
    my $pattern = $svgDocument->createElement("pattern");
    my $use1    = $svgDocument->createElement("use");
    my $use2    = $svgDocument->createElement("use");

    $pattern->setAttribute("id", "tile");
    $pattern->setAttribute("height", $pattern_height);
    $pattern->setAttribute("width", $pattern_width);
    $pattern->setAttribute("patternUnits", "userSpaceOnUse");
    $pattern->setAttribute("overflow", "visible");

    $use1->setAttribute("xlink:href", "#supercluster");
    $use1->setAttribute(
        "transform",
        "translate(0,$pattern_half_height)"
    );
    $pattern->appendChild($use1);

    $use2->setAttribute("xlink:href", "#supercluster");
    $use2->setAttribute(
        "transform",
        "translate($pattern_width,$pattern_half_height) scale(-1,1)"
    );
    $pattern->appendChild($use2);

    return $pattern;
}


#####
#
#   create_filter
#
#####
sub create_filter {
    my $filter = $svgDocument->createElement("filter");

    $filter->setAttribute("id", "dropShadow");
    $filter->setAttribute("height", "130%");
    $filter->setAttribute("width", "130%");

    $filter->appendChild( create_gaussian_blur()      );
    $filter->appendChild( create_offset()             );
    $filter->appendChild( create_component_transfer() );
    $filter->appendChild( create_merge()              );

    return $filter;
}


#####
#
#   create_gaussian_blur
#
#####
sub create_gaussian_blur {
    my $gaussian = $svgDocument->createElement("feGaussianBlur");
    
    $gaussian->setAttribute("in", "SourceAlpha");
    $gaussian->setAttribute("stdDeviation", $blur_amount);
    $gaussian->setAttribute("result", "shadow");

    return $gaussian;
}


#####
#
#   create_offset
#
#####
sub create_offset {
    my $offset = $svgDocument->createElement("feOffset");

    $offset->setAttribute("in", "shadow");
    $offset->setAttribute("dx", $blur_offset_x);
    $offset->setAttribute("dy", $blur_offset_y);
    $offset->setAttribute("result", "offsetShadow");

    return $offset;
}


#####
#
#   create_component_transfer
#
#####
sub create_component_transfer {
    my $transfer   = $svgDocument->createElement("feComponentTransfer");
    my $funcR      = $svgDocument->createElement("feFuncR");
    my $funcG      = $svgDocument->createElement("feFuncG");
    my $funcB      = $svgDocument->createElement("feFuncB");

    $transfer->setAttribute("in", "offsetShadow");
    $transfer->setAttribute("result", "lighterOffsetShadow");

    $funcR->setAttribute("type", "linear");
    $funcR->setAttribute("intercept", $blur_lighten_amount);
    $transfer->appendChild($funcR);

    $funcG->setAttribute("type", "linear");
    $funcG->setAttribute("intercept", $blur_lighten_amount);
    $transfer->appendChild($funcG);

    $funcB->setAttribute("type", "linear");
    $funcB->setAttribute("intercept", $blur_lighten_amount);
    $transfer->appendChild($funcB);

    return $transfer;
}


#####
#
#   create_merge
#
#####
sub create_merge {
    my $merge      = $svgDocument->createElement("feMerge");
    my $mergeNode1 = $svgDocument->createElement("feMergeNode");
    my $mergeNode2 = $svgDocument->createElement("feMergeNode");

    $mergeNode1->setAttribute("in", "lighterOffsetShadow");
    $merge->appendChild($mergeNode1);

    $mergeNode2->setAttribute("in", "SourceGraphic");
    $merge->appendChild($mergeNode2);

    return $merge;
}


#####
#
#   create_quote
#
#####
sub create_quote {
    my $g     = $svgDocument->createElement("g");
    my $desc  = $svgDocument->createElement("desc");
    my $tnode = $svgDocument->createTextNode("A quote with a tiled background");
    
    $desc->appendChild($tnode);
    $g->appendChild($desc);
    
    $g->appendChild( create_background() );
    $g->appendChild( create_text()       );

    return $g;
}


#####
#
#   create_background
#
#####
sub create_background {
    my $g                  = $svgDocument->createElement("g");
    my $desc               = $svgDocument->createElement("desc");
    my $tnode              = $svgDocument->createTextNode("The tiled background");
    my $sidebar_background = $svgDocument->createElement("rect");
    my $sidebar_tile       = $svgDocument->createElement("rect");
    my $background         = $svgDocument->createElement("rect");
    
    $desc->appendChild($tnode);
    $g->appendChild($desc);

    $sidebar_background->setAttribute("width", $sidebar_width);
    $sidebar_background->setAttribute("height", "100%");
    $sidebar_background->setAttribute("fill", $sidebar_background_color);
    $g->appendChild($sidebar_background);

    $sidebar_tile->setAttribute("width", $sidebar_width);
    $sidebar_tile->setAttribute("height", "100%");
    $sidebar_tile->setAttribute("fill", "url(#tile)");
    $g->appendChild($sidebar_tile);

    $background->setAttribute("x", $sidebar_width);
    $background->setAttribute("width", "100%");
    $background->setAttribute("height", "100%");
    $background->setAttribute("opacity", $background_opacity);
    $background->setAttribute("fill", "url(#tile)");
    $g->appendChild($background);

    return $g;
}


#####
#
#   create_text
#
#####
sub create_text {
    my $text   = $svgDocument->createElement("text");
    my $desc   = $svgDocument->createElement("desc");
    my $tnode  = $svgDocument->createTextNode("The quote");
    my $index  = generate_integer( 0, @$quotes - 1 );
    my $quote  = $quotes->[$index];
    my $author = $quote->{author};
    my $lines  = $quote->{lines};

    $desc->appendChild($tnode);
    $text->appendChild($desc);

    $text->setAttribute("class", "quote");
    $text->setAttribute("y", $quote_top_margin);
    $text->setAttribute("fill", $quote_color);
    $text->setAttribute("filter", "url(#dropShadow)");
    
    foreach ( @$lines ) {
        my $tspan = $svgDocument->createElement("tspan");
        my $tnode = $svgDocument->createTextNode($_);

        $tspan->setAttribute("x", $quote_left_margin);
        $tspan->setAttribute("dy", "1em");
        $tspan->appendChild($tnode);

        $text->appendChild($tspan);
    }

    my $tspan = $svgDocument->createElement("tspan");

    $tnode = $svgDocument->createTextNode($author);

    $tspan->setAttribute("class", "author");
    $tspan->setAttribute("x", $quote_left_margin);
    $tspan->setAttribute("dy", "2em");
    $tspan->appendChild($tnode);

    $text->appendChild($tspan);

    return $text;
}

