#!/usr/bin/perl -w
#####
#
#   svg_via_svgpm.pl
#   written by Kevin Lindsey (kevin@kevlindev.com)
#
#####

#####
#
#   includes
#
#####
use QOTDCommon;
use SVG;


#####
#
#   globals
#
#####
my $svgDocument;
my $svgRoot;


#####
#
#   main
#
#####
create_SVG_Document();
create_style($svgRoot);
create_defs($svgRoot);
create_quote($svgRoot);

print "Pragma: no-cache\n";
print "Content-type: image/svg+xml\n\n";
print $svgDocument->xmlify();


#####
#
#   create_SVG_Document
#
#####
sub create_SVG_Document {
    $svgDocument = new SVG(
        "xmlns"       => "http://www.w3.org/2000/svg",
        "xmlns:xlink" => "http://www.w3.org/1999/xlink",
        -standalone   => "no",
        -encoding     => "ISO-8859-1",
        -indent       => "    "
    );

    $svgRoot = $svgDocument->{-childs}[0];
    #$svgRoot->{'viewBox'} = "0 0 100 100";
    delete $svgRoot->{width};
    delete $svgRoot->{height};
}


#####
#
#   create_style
#
#####
sub create_style {
    my $parent = shift;
    my $style  = $parent->tag("style", type => "text/css");
    my $cdata  = $style->CDATA(<<EOL);
    .quote { font-size: 28; font-weight: bold }
    .author { font-size: 24; font-weight: normal; font-style: italic }
EOL
}


#####
#
#   create_defs
#
#####
sub create_defs {
    my $parent = shift;
    my $defs   = $svgRoot->defs();
    
    create_tile_element($defs);
    create_cluster($defs);
    create_supercluster($defs);
    create_pattern($defs);
    create_filter($defs);
}


#####
#
#   create_tile_element
#
#####
sub create_tile_element {
    my $parent = shift;

    $parent->path(
        id             => "elem",
        d              => $path_data,
        stroke         => $path_stroke_color,
        "stroke-width" => $path_stroke_width,
        fill           => $path_fill_color,
        "fill-opacity" => $path_fill_opacity,
        transform      => $path_transform
    );
}


#####
#
#   create_cluster
#
#####
sub create_cluster {
    my $parent = shift;
    my $g      = $parent->group(id => "cluster");

    $g->use(
        "xlink:href" => "#elem",
        transform    => "rotate(0)"
    );

    $g->use(
        "xlink:href" => "#elem",
        transform    => "rotate(120)"
    );

    $g->use(
        "xlink:href" => "#elem",
        transform    => "rotate(240)"
    );
}


#####
#
#   create_supercluster
#
#####
sub create_supercluster {
    my $parent = shift;
    my $g      = $parent->group(id => "supercluster");

    $g->use(
        "xlink:href" => "#cluster",
        transform    => "rotate(60) translate($hexagon_radius,0) rotate(60) scale(-1,1)"
    );

    $g->use(
        "xlink:href" => "#cluster",
        transform    => "rotate(0) translate($hexagon_radius,0)"
    );

    $g->use(
        "xlink:href" => "#cluster",
        transform    => "rotate(-60) translate($hexagon_radius,0) scale(1,-1)"
    );
}


#####
#
#   create_pattern
#
#####
sub create_pattern {
    my $parent  = shift;
    my $pattern = $parent->pattern(
        id           => "tile",
        height       => $pattern_height,
        width        => $pattern_width,
        patternUnits => "userSpaceOnUse",
        overflow     => "visible"
    );

    $pattern->use(
        "xlink:href" => "#supercluster",
        transform    => "translate(0,$pattern_half_height)"
    );

    $pattern->use(
        "xlink:href" => "#supercluster",
        transform    => "translate($pattern_width,$pattern_half_height) scale(-1,1)"
    );
}


#####
#
#   create_filter
#
#####
sub create_filter {
    my $parent = shift;
    my $filter = $parent->filter(
        id     => "dropShadow",
        height => "130%",
        width  => "130%"
    );

    create_gaussian_blur($filter);
    create_offset($filter);
    create_component_transfer($filter);
    create_merge($filter);
}


#####
#
#   create_gaussian_blur
#
#####
sub create_gaussian_blur {
    my $parent = shift;

    $parent->tag(
        "feGaussianBlur",
        in           => "SourceAlpha",
        stdDeviation => $blur_amount,
        result       => "shadow"
    );
}


#####
#
#   create_offset
#
#####
sub create_offset {
    my $parent = shift;

    $parent->tag(
        "feOffset",
        in     => "shadow",
        dx     => $blur_offset_x,
        dy     => $blur_offset_y,
        result => "offsetShadow"
    );
}


#####
#
#   create_component_transfer
#
#####
sub create_component_transfer {
    my $parent   = shift;
    my $transfer = $parent->tag(
        "feComponentTransfer",
        in     => "offsetShadow",
        result => "lighterOffsetShadow"
    );

    $transfer->tag(
        "feFuncR",
        type      => "linear",
        intercept => $blur_lighten_amount
    );

    $transfer->tag(
        "feFuncG",
        type      => "linear",
        intercept => $blur_lighten_amount
    );

    $transfer->tag(
        "feFuncB",
        type      => "linear",
        intercept => $blur_lighten_amount
    );
}


#####
#
#   create_merge
#
#####
sub create_merge {
    my $parent = shift;
    my $merge  = $parent->tag("feMerge");

    $merge->tag(
        "feMergeNode",
        in => "lighterOffsetShadow"
    );

    $merge->tag(
        "feMergeNode",
        in => "SourceGraphic"
    );
}


#####
#
#   create_quote
#
#####
sub create_quote {
    my $parent = shift;
    my $g      = $parent->group();
    
    $g->desc()->cdata("A quote with a tiled background");
    
    create_background($g);
    create_text($g);
}


#####
#
#   create_background
#
#####
sub create_background {
    my $parent = shift;
    my $g      = $parent->group();
    
    $g->desc()->cdata("The tiled background");
    $g->rect(
        width  => $sidebar_width,
        height => "100%",
        fill   => $sidebar_background_color
    );
    $g->rect(
        width  => $sidebar_width,
        height => "100%",
        fill   => "url(#tile)"
    );
    $g->rect(
        x       => $sidebar_width,
        width   => "100%",
        height  => "100%",
        opacity => $background_opacity,
        fill    => "url(#tile)"
    );
}


#####
#
#   create_text
#
#####
sub create_text {
    my $parent = shift;
    my $text   = $parent->text(
        class  => "quote",
        y      => $quote_top_margin,
        fill   => $quote_color,
        filter => "url(#dropShadow)"
    );

    $text->desc()->cdata("The quote");

    my $index  = generate_integer( 0, @$quotes - 1 );
    my $quote  = $quotes->[$index];
    my $author = $quote->{author};
    my $lines  = $quote->{lines};

    foreach ( @$lines ) {
        $text->text(
            -type => "span",
            x     => $quote_left_margin,
            dy    => "1em"
        )->cdata($_);
    }

    $text->text(
        -type => "span",
        x     => $quote_left_margin,
        dy    => "2em",
        class => "author"
    )->cdata($author);
}

